# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class absencePerContract(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.absencePerContract'
	def __init__(self, cr, uid, name, context):
		super(absencePerContract, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'getLoadRegisters': self._get_load_registers,
			'startDate': self._startDate,
			'endDate': self._endDate,
		})
		
	def _loadRegisters(self, form):
		contracts_ids = form['contracts_ids']
		contracts_ids = contracts_ids[0][2] #gets the ids selected in the many2many field
		contracts_ids = self.pool.get('hr.contract').search(self.cr, self.uid,[('id','in',contracts_ids)], order='code, date_start')
		debug(contracts_ids)
		#to store the answer
		answ = []
		for contract_id in contracts_ids:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			contractCode = contract.code
			
			employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
			employee_name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
			#charge the absence register whit all the information
			registerAbs = {'contractCode':contractCode, 'employee_name':employee_name, 'contract_id':contract_id} 
			#append the register to the answere
			answ.append(registerAbs)
		#return the list of absence	
		return answ
	
	def _get_load_registers(self, contract_id, form):
		#to store the answer
		answ = []
		contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
		#search for each absnce per contract
		absences = self.pool.get('hr_payroll_cr.absence').search(self.cr, self.uid,[('contract','=',contract_id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd'])], order='date')
		#for each absence per contract
		for absence_id in absences:
			absence = self.pool.get('hr_payroll_cr.absence').browse(self.cr, self.uid, absence_id)#loads the absence register to store the absence data
			payment_per_contract_line_browse = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, absence.payment_per_contract_line.id)
			#charge the absence register whit all the information
			if(absence.justification):
				registerAbs = {'detail':absence.detail, 'date':absence.date, 'justify':'Si', 'amount':payment_per_contract_line_browse.amount}
			else:
				registerAbs = {'detail':absence.detail, 'date':absence.date, 'justify':'No', 'amount':payment_per_contract_line_browse.amount}
			#append the register to the answer
			answ.append(registerAbs)
		#return the list of absence	
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		

report_sxw.report_sxw('report.hr_payroll_cr.absencePerContract', 'hr_payroll_cr.absence', 'addons/hr_payroll_cr/report/absencePerContract.rml', parser=absencePerContract, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
