# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class commissionPerCompanyTotal(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.commissionPerCompanyTotal'
	def __init__(self, cr, uid, name, context):
		super(commissionPerCompanyTotal, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'startDate': self._startDate,
			'endDate': self._endDate,
		})
		
	def _loadRegisters(self, form):
		company_id = form['company_id']
		#to store the answer
		answ = []
		#search for all the employees of the given company
		contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id)], order='code')
		employee_list = []		
		for contract_id in contracts:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			employee_list.append(contract.employee_id.id)
		employees = self.pool.get('hr.employee').search(self.cr, self.uid,[('id','in',employee_list)], order='surname1, surname2, name')
		commissions = self.pool.get('hr_payroll_cr.commission').search(self.cr, self.uid,[('contract','in',contracts), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd'])], order='date')

		if form['order'] == '1':
			for contract_id in contracts:
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
				contractCode = contract.code			
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				
				commissions = self.pool.get('hr_payroll_cr.commission').search(self.cr, self.uid,[('contract','=',contract_id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd'])], order='date')
				commission_count = 0
				commission_amount = 0
				#for each commission per contract
				for commission_id in commissions:
					commission = self.pool.get('hr_payroll_cr.commission').browse(self.cr, self.uid, commission_id)#loads the over_salary register to store the over_salary data
					if commission.payment_per_contract_line.id:
						payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, commission.payment_per_contract_line.id)
						payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contract_line.payment_per_contract.id)
						commission_count += 1
						commission_amount += payment_per_contract_line.amount
				#charge the commission register with all the information
				register = {'name':name, 'contractCode':contractCode, 'amount':commission_amount, 'count':commission_count} 
				#append the register to the answer
				answ.append(register)
					
		elif form['order'] == '2':
			for employee_id in employees:
				contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id),('employee_id','=',employee_id)], order='date_start, code')
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, employee_id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
							
				for contract_id in contracts:
					contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
					contractCode = contract.code
					commissions = self.pool.get('hr_payroll_cr.commission').search(self.cr, self.uid,[('contract','=',contract_id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd'])], order='date')
					commission_count = 0
					commission_amount = 0
					#for each commission per contract
					for commission_id in commissions:
						commission = self.pool.get('hr_payroll_cr.commission').browse(self.cr, self.uid, commission_id)#loads the over_salary register to store the over_salary data
						if commission.payment_per_contract_line.id:
							payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, commission.payment_per_contract_line.id)
							payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contract_line.payment_per_contract.id)
							commission_count += 1
							commission_amount += payment_per_contract_line.amount
					#charge the commission register with all the information
					register = {'name':name, 'contractCode':contractCode, 'amount':commission_amount, 'count': commission_count} 
					#append the register to the answer
					answ.append(register)			
		else:
			#for each over_salary per contract
			for commission_id in commissions:
				commission = self.pool.get('hr_payroll_cr.commission').browse(self.cr, self.uid, commission_id)#loads the over_salary register to store the over_salary data
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, commission.contract.id)#loads the contract register to store the contract name
				contractCode = contract.code				
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				
				paid_date = ""
				if commission.payment_per_contract_line.id:
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, commission.payment_per_contract_line.id)
					payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contract_line.payment_per_contract.id)
					paid_date = payment_per_contract.date
				else:
					paid_date = 'No pagada'
				#charge the over_salary register with all the information
				if (commission.percent):
					register = {'name':name, 'contractCode':contractCode, 'detail':commission.detail, 'date':commission.date, 'amount':commission.amount, 'paid_date':paid_date, 'percent': 'Sí'} 
				else: 
					register = {'name':name, 'contractCode':contractCode, 'detail':commission.detail, 'date':commission.date, 'amount':commission.amount, 'paid_date':paid_date, 'percent': 'No'} 
				#append the register to the answer
				answ.append(register)
		#return the list of over_salary	
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		

report_sxw.report_sxw('report.hr_payroll_cr.commissionPerCompanyTotal', 'hr_payroll_cr.commission', 'addons/hr_payroll_cr/report/commissionPerCompanyTotal.rml', parser=commissionPerCompanyTotal, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
