# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class commissionPerContract(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.commissionPerContract'
	def __init__(self, cr, uid, name, context):
		super(commissionPerContract, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'getLoadRegisters': self._get_load_registers,
			'startDate': self._startDate,
			'endDate': self._endDate,
		})
		
	def _loadRegisters(self, form):
		contracts_ids = form['contracts_ids']
		contracts_ids = contracts_ids[0][2] #gets the ids selected in the many2many field
		contracts_ids = self.pool.get('hr.contract').search(self.cr, self.uid,[('id','in',contracts_ids)], order='code, date_start')
		debug(contracts_ids)
		#to store the answere
		answ = []
		for contract_id in contracts_ids:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			contractCode = contract.code
			
			employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
			employee_name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
			debug(employee_name)
			#charge the over_salary register whit all the information
			register = {'contractCode':contractCode, 'employee_name':employee_name, 'contract_id':contract_id} 
			#append the register to the answere
			answ.append(register)
		#return the list of over_salary	
		return answ
	
	def _get_load_registers(self, contract_id, form):
		#to store the answere
		debug(contract_id)
		answ = []
		contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
		#search for each commission per contract
		commissions = self.pool.get('hr_payroll_cr.commission').search(self.cr, self.uid,[('contract','=',contract_id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd'])], order='date')
		#for each commission per contract
		for commission_id in commissions:
			commission = self.pool.get('hr_payroll_cr.commission').browse(self.cr, self.uid, commission_id)#loads the commission register to store the commission data
			#charge the commission register whit all the information
			amount = ""
			if (commission.percent):
				register = {'detail':commission.detail, 'date':commission.date, 'amount':commission.amount, 'percent': 'Sí'} 
			else: 
				register = {'detail':commission.detail, 'date':commission.date, 'amount':commission.amount, 'percent': 'No'} 		
			#append the register to the answer
			answ.append(register)
		#return the list of commission
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		

report_sxw.report_sxw('report.hr_payroll_cr.commissionPerContract', 'hr_payroll_cr.commission', 'addons/hr_payroll_cr/report/commissionPerContract.rml', parser=commissionPerContract, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
