# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
import calendar
from report import report_sxw
import tools
from tools import debug

class deductionPerCompany(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.deductionPerCompany'
	def __init__(self, cr, uid, name, context):
		super(deductionPerCompany, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'month': self._month,
			'year': self._year,
			'deduction_type': self._deduction_type,
			'get_company': self._get_company,
		})
		
	def _loadRegisters(self, form):
		company_id = form['company_id']
		deduction_type_id = form['deduction_type']
		#to store the answer
		answ = []
		date_start = self._date_start(form)
		date_end = self._date_end(form)
		#search for all the employees of the given company
		contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id)], order='code')
		employee_list = []
		for contract_id in contracts:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			employee_list.append(contract.employee_id.id)
		employees = self.pool.get('hr.employee').search(self.cr, self.uid,[('id','in',employee_list)], order='surname1, surname2, name')
		deduction_type = self.pool.get('hr_payroll_cr.deduction_type').search(self.cr,self.uid,[('id','=',deduction_type_id)])
		if form['order']=='1':
			for contract_id in contracts:
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
				contractCode = contract.code
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				deductions = self.pool.get('hr_payroll_cr.deduction').search(self.cr , self.uid, [('contract','=',contract_id), ('deduction_type','=',deduction_type[0])])
				for deduction_id in deductions:
					deduction = self.pool.get('hr_payroll_cr.deduction').browse(self.cr, self.uid, deduction_id)
					deduction_lines = self.pool.get('hr_payroll_cr.deduction_line').search(self.cr, self.uid,[('date','>=',date_start), ('date','<=',date_end), ('deduction','=',deduction_id)])
					for deduction_line in deduction_lines:
						deduction_line_browse = self.pool.get('hr_payroll_cr.deduction_line').browse(self.cr, self.uid, deduction_line)
						payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr,self.uid, [('id','=',deduction_line_browse.payment_per_contract_line.id)])
						payment_per_contract_line_browse = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr,self.uid, payment_per_contract_line[0])
						payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('id','=',payment_per_contract_line_browse.payment_per_contract.id),('date','>=',date_start),('date','<=',date_end)])
						payment_per_contract_browse = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contract[0])
						register = {'name':name, 'contractCode':contractCode, 'date':payment_per_contract_browse.date, 'deduction_amount':payment_per_contract_line_browse.amount} 
						#append the register to the answer
						answ.append(register)
		elif form['order'] == '2':
			for employee_id in employees:
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, employee_id)
				contracts = self.pool.get('hr.contract').search(self.cr, self.uid, [('company_id','=',company_id),('employee_id','=',employee_id)], order = 'date_start, code')
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				for contract_id in contracts:
					contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)
					contractCode = contract.code
					deductions = self.pool.get('hr_payroll_cr.deduction').search(self.cr, self.uid, [('contract','=',contract_id), ('deduction_type','=',deduction_type[0])])
					deduction_lines = self.pool.get('hr_payroll_cr.deduction_line').search(self.cr, self.uid, [('date','>=',date_start), ('date','<=',date_end), ('deduction','in',deductions)], order='date')
					debug(deduction_lines)
					for deduction_line in deduction_lines:
						deduction_line_browse = self.pool.get('hr_payroll_cr.deduction_line').browse(self.cr, self.uid, deduction_line)
						payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('id','=',deduction_line_browse.payment_per_contract_line.id)])
						payment_per_contract_line_browse = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr,self.uid, payment_per_contract_line[0])
						payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('id','=',payment_per_contract_line_browse.payment_per_contract.id),('date','>=',date_start),('date','<=',date_end)])
						payment_per_contract_browse = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contract[0])
						register = {'name':name, 'contractCode':contractCode, 'date':payment_per_contract_browse.date, 'deduction_amount':payment_per_contract_line_browse.amount} 
						#append the register to the answer
						answ.append(register)
		else:
			
			deductions = self.pool.get('hr_payroll_cr.deduction').search(self.cr , self.uid, [('contract','in',contracts), ('deduction_type','=',deduction_type[0])])
			deduction_lines = self.pool.get('hr_payroll_cr.deduction_line').search(self.cr, self.uid, [('date','>=',date_start), ('date','<=',date_end), ('deduction','in',deductions)], order='date')
			for deduction_line in deduction_lines:
				deduction_line_browse = self.pool.get('hr_payroll_cr.deduction_line').browse(self.cr, self.uid, deduction_line)
				deduction = self.pool.get('hr_payroll_cr.deduction').search(self.cr , self.uid, [('id','=', deduction_line_browse.deduction.id)])
				deduction_browse = self.pool.get('hr_payroll_cr.deduction').browse(self.cr, self.uid, deduction[0])
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, deduction_browse.contract.id)
				contractCode = contract.code
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id)
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('id','=',deduction_line_browse.payment_per_contract_line.id)])
				payment_per_contract_line_browse = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr,self.uid, payment_per_contract_line[0])
				payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('id','=',payment_per_contract_line_browse.payment_per_contract.id),('date','>=',date_start),('date','<=',date_end)])
				payment_per_contract_browse = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contract[0])
				
				register = {'name':name, 'contractCode':contractCode, 'date':payment_per_contract_browse.date, 'deduction_amount':payment_per_contract_line_browse.amount} 
				#append the register to the answer
				answ.append(register)
				
		#return the list of over_salary	
		return answ
				
	def _year(self, form):
		year = form['year']
		return year

	def _deduction_type(self, form):
		type_id = form['deduction_type']
		type = self.pool.get('hr_payroll_cr.deduction_type').browse(self.cr, self.uid, type_id)
		debug(type.name)
		return type.name
		
	def _get_company(self, form):
		company_id = form['company_id']
		debug(company_id)
		name = self.pool.get('res.company').browse(self.cr, self.uid, company_id)
		debug(name)
		debug(name.name)
		return name.name		
		
	#TO DO: eliminate months of the code (months in spanish)	
	def _month(self, form):
		months = {'1':'Enero', '2':'Febrero', '3':'Marzo', '4':'Abril', '5':'Mayo', '6':'Junio', '7':'Julio',
				'8':'Agosto', '9':'Setiembre', '10':'Octubre', '11':'Noviembre', '12':'Diciembre'}
		return months[form['month']]
	
	def _date_start(self, form):
		month = int(form['month'])
		year = int(form['year'])
		return datetime.date(year, month, 01)

	def _date_end(self, form):
		month = int(form['month'])
		year = int(form['year'])
		day = int(calendar.monthrange(year, month)[1])
		return datetime.date(year, month, day)

report_sxw.report_sxw('report.hr_payroll_cr.deductionPerCompany', 'hr_payroll_cr.deduction', 'addons/hr_payroll_cr/report/deductionPerCompany.rml', parser=deductionPerCompany, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
