# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class foreclosurePerCompanyTotal(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.foreclosurePerCompanyTotal'
	def __init__(self, cr, uid, name, context):
		super(foreclosurePerCompanyTotal, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'startDate': self._startDate,
			'endDate': self._endDate,
			'loadRegisters':self._loadRegisters,
			'get_company': self._get_company,
			'foreclosure_type': self._foreclosure_type,
		})
		
	def _loadRegisters(self, form):
		company_id = form['company_id']
		#to store the answere
		answ = []
		foreclosure_type_id = form['foreclosure_type_id']
		#search for all the employees of the given company
		contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id)], order='code')
		employee_list = []
		for contract_id in contracts:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			employee_list.append(contract.employee_id.id)
		employees = self.pool.get('hr.employee').search(self.cr, self.uid,[('id','in',employee_list)], order='surname1, surname2, name')
		foreclosure_type_browse = self.pool.get('hr_payroll_cr.foreclosure_type').browse(self.cr, self.uid, foreclosure_type_id)
		foreclosure_type = foreclosure_type_browse.name
		foreclosures = self.pool.get('hr_payroll_cr.foreclosure').search(self.cr, self.uid,[('contract','in',contracts), ('start_date','>=',form['dateStart']), ('start_date','<=',form['dateEnd']), ('foreclosure_type','=',foreclosure_type)], order='start_date')
		
		if form['order'] == '1':
			for contract_id in contracts:
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
				contractCode = contract.code			
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				foreclosures = self.pool.get('hr_payroll_cr.foreclosure').search(self.cr, self.uid,[('contract','=',contract_id), ('foreclosure_type','=',foreclosure_type)])
				amount_total = 0
				#for each foreclosure per contract
				for foreclosure_id in foreclosures:
					foreclosure = self.pool.get('hr_payroll_cr.foreclosure').browse(self.cr, self.uid, foreclosure_id)#loads the foreclosure register to store the foreclosure data			
					foreclosure_payment = self.pool.get('hr_payroll_cr.foreclosure_payment').search(self.cr, self.uid,[('foreclosure','=',foreclosure.id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd']),])
					for foreclosure_payment_id in foreclosure_payment:
						foreclosure_browse = self.pool.get('hr_payroll_cr.foreclosure_payment').browse(self.cr, self.uid,foreclosure_payment_id)
						foreclosure_browse_payment = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid,foreclosure_browse.payment_per_contract_line.id)
						amount_total += foreclosure_browse_payment.amount
				if (amount_total != 0):
					#charge the foreclosure register whit all the information
					register = {'name':name, 'contractCode':contractCode, 'amount':amount_total} 
					#append the register to the answer
					answ.append(register)
					
		elif form['order'] == '2':
			for employee_id in employees:
				contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id),('employee_id','=',employee_id)], order='date_start, code')
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, employee_id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
							
				for contract_id in contracts:
					contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
					contractCode = contract.code
					foreclosures = self.pool.get('hr_payroll_cr.foreclosure').search(self.cr, self.uid,[('contract','=',contract_id), ('foreclosure_type','=',foreclosure_type)], order='start_date')
					amount_total = 0
					#for each foreclosure per contract
					for foreclosure_id in foreclosures:
						foreclosure = self.pool.get('hr_payroll_cr.foreclosure').browse(self.cr, self.uid, foreclosure_id)#loads the foreclosure register to store the foreclosure data
						foreclosure_payment = self.pool.get('hr_payroll_cr.foreclosure_payment').search(self.cr, self.uid,[('foreclosure','=',foreclosure.id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd'])])

						for foreclosure_payment_id in foreclosure_payment:
							foreclosure_browse = self.pool.get('hr_payroll_cr.foreclosure_payment').browse(self.cr, self.uid,foreclosure_payment_id)
							foreclosure_browse_payment = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid,foreclosure_browse.payment_per_contract_line.id)
							amount_total += foreclosure_browse_payment.amount
					if (amount_total != 0):
						#charge the foreclosure register whit all the information
						register = {'name':name, 'contractCode':contractCode, 'amount':amount_total} 
						#append the register to the answer
						answ.append(register)				
		#return the list of foreclosures	
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		
	def _get_company(self, form):
		company_id = form['company_id']
		debug(company_id)
		name = self.pool.get('res.company').browse(self.cr, self.uid, company_id)
		debug(name)
		debug(name.name)
		return name.name
		
	def _foreclosure_type(self, form):
		type_id = form['foreclosure_type_id']
		type = self.pool.get('hr_payroll_cr.foreclosure_type').browse(self.cr, self.uid, type_id)
		debug(type.name)
		return type.name
		

report_sxw.report_sxw('report.hr_payroll_cr.foreclosurePerCompanyTotal', 'hr_payroll_cr.foreclosure', 'addons/hr_payroll_cr/report/foreclosurePerCompanyTotal.rml', parser=foreclosurePerCompanyTotal, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
