# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class foreclosurePerContract(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.foreclosurePerContract'
	def __init__(self, cr, uid, name, context):
		super(foreclosurePerContract, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'getLoadRegisters': self._get_load_registers,
			'startDate': self._startDate,
			'endDate': self._endDate,
		})
		
	def _loadRegisters(self, form):
		contracts_ids = form['contracts_ids']
		debug(contracts_ids)
		contracts_ids = contracts_ids[0][2] #gets the ids selected in the many2many field
		contracts_ids = self.pool.get('hr.contract').search(self.cr, self.uid,[('id','in',contracts_ids)], order='code, date_start')
		#to store the answere
		answ = []
		for contract_id in contracts_ids:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			contractCode = contract.code
			
			employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
			employee_name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name

			#charge the over_salary register whit all the information
			registerAbs = {'contractCode':contractCode, 'employee_name':employee_name, 'contract_id':contract_id} 
			#append the register to the answere
			answ.append(registerAbs)
		#return the list of over_salary	
		return answ
	
	def _get_load_registers(self, contract_id, form):
		#to store the answere
		debug(contract_id)
		answ = []
		contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
		#search for each absnce per contract
		foreclosures = self.pool.get('hr_payroll_cr.foreclosure').search(self.cr, self.uid,[('contract','=',contract_id), ('start_date','>=',form['dateStart']), ('start_date','<=',form['dateEnd'])], order='start_date')
		#for each over_salary per contract
		for foreclosure_id in foreclosures:
			foreclosure = self.pool.get('hr_payroll_cr.foreclosure').browse(self.cr, self.uid, foreclosure_id)#loads the over_salary register to store the over_salary data
			#charge the over_salary register whit all the information
			
			debug(foreclosure.detail)
			
			foreclosure_payment = self.pool.get('hr_payroll_cr.foreclosure_payment').search(self.cr, self.uid,[('foreclosure','=',foreclosure.id), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd']),])
					
			for foreclosure_payment_id in foreclosure_payment:
				foreclosure_browse = self.pool.get('hr_payroll_cr.foreclosure_payment').browse(self.cr, self.uid,foreclosure_payment_id)
				foreclosuretype = self.pool.get('hr_payroll_cr.foreclosure_type').browse(self.cr, self.uid,foreclosure.foreclosure_type.id)
				foreclosure_browse_payment = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid,foreclosure_browse.payment_per_contract_line.id)
			
				debug(foreclosure_browse)
				debug(foreclosure_browse_payment)
				
				registerAbs = {'type':foreclosuretype.name, 'detail':foreclosure.detail, 'date':foreclosure_browse.date, 'amount':foreclosure_browse_payment.amount}
			
			#append the register to the answere
			answ.append(registerAbs)
		#return the list of over_salary	
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return date
		
	def _endDate(self, form):
		date = form['dateEnd']
		return date
		

report_sxw.report_sxw('report.hr_payroll_cr.foreclosurePerContract', 'hr_payroll_cr.foreclosure', 'addons/hr_payroll_cr/report/foreclosurePerContract.rml', parser=foreclosurePerContract, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
