# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class incapacityPeriodPerCompanyTotal(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.incapacityPeriodPerCompanyTotal'
	def __init__(self, cr, uid, name, context):
		super(incapacityPeriodPerCompanyTotal, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'startDate': self._startDate,
			'endDate': self._endDate,
		})
		
	def _loadRegisters(self, form):
		company_id = form['company_id']
		#to store the answer
		answ = []
		#search for all the employees of the given company
		contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id)], order='code')
		employee_list = []
		for contract_id in contracts:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			employee_list.append(contract.employee_id.id)
		employees = self.pool.get('hr.employee').search(self.cr, self.uid,[('id','in',employee_list)], order='surname1, surname2, name')
		incapacity_periods = self.pool.get('hr_payroll_cr.incapacity_period').search(self.cr, self.uid,[('contract','in',contracts), ('start_date','>=',form['dateStart']), ('end_date','<=',form['dateEnd'])], order='start_date')
		if form['order'] == '1':
			for contract_id in contracts:
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
				contractCode = contract.code			
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				paid_days_total_per_contract = 0
				days_to_pay_total_per_contract = 0
				total_paid_per_contract = 0
				#for each incapacity_period per contract
				for incapacity_period_id in incapacity_periods:
					incapacity_period = self.pool.get('hr_payroll_cr.incapacity_period').browse(self.cr, self.uid, incapacity_period_id)#loads the incapacity_period register to store the incapacity_period data
					paid_days = self.pool.get('hr_payroll_cr.incapacity_period_per_day').search(self.cr, self.uid, [('incapacity_period', '=', incapacity_period_id), ('paid', '=', True)])
					days_to_pay = self.pool.get('hr_payroll_cr.incapacity_period_per_day').search(self.cr, self.uid, [('incapacity_period', '=', incapacity_period_id), ('paid', '=', False)])
					paid_days_total = 0
					days_to_pay_total = 0
					total_paid = 0
					for pd in paid_days:
						pd_browse = self.pool.get('hr_payroll_cr.incapacity_period_per_day').browse(self.cr, self.uid, pd)
						payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, pd_browse.payment_per_contract_line.id)
						paid_days_total = paid_days_total + 1
						total_paid = total_paid + payment_per_contract_line.amount
					for dtp in days_to_pay:
						days_to_pay_total = days_to_pay_total + 1
					paid_days_total_per_contract += paid_days_total
					days_to_pay_total_per_contract += days_to_pay_total
					total_paid_per_contract += total_paid
					#charge the incapacity_period register whit all the information
					register = {'total_days': paid_days_total_per_contract + days_to_pay_total_per_contract, 'days_to_pay':days_to_pay_total_per_contract, 'paid_days':paid_days_total_per_contract, 'total':total_paid_per_contract, 'name':name, 'contractCode':contractCode} 
					#append the register to the answere
					answ.append(register)
		elif form['order'] == '2':
			for employee_id in employees:
				contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id),('employee_id','=',employee_id)], order='date_start, code')
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, employee_id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
							
				for contract_id in contracts:
					contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
					contractCode = contract.code
					incapacity_periods = self.pool.get('hr_payroll_cr.incapacity_period').search(self.cr, self.uid,[('contract','=',contract_id), ('start_date','>=',form['dateStart']), ('end_date','<=',form['dateEnd'])], order='start_date')
					if (incapacity_periods != []):
						paid_days_total_per_contract = 0
						days_to_pay_total_per_contract = 0
						total_paid_per_contract = 0
						#for each incapacity_period per contract
						for incapacity_period_id in incapacity_periods:
							incapacity_period = self.pool.get('hr_payroll_cr.incapacity_period').browse(self.cr, self.uid, incapacity_period_id)#loads the incapacity_period register to store the incapacity_period data
							paid_days = self.pool.get('hr_payroll_cr.incapacity_period_per_day').search(self.cr, self.uid, [('incapacity_period', '=', incapacity_period_id), ('paid', '=', True)])
							days_to_pay = self.pool.get('hr_payroll_cr.incapacity_period_per_day').search(self.cr, self.uid, [('incapacity_period', '=', incapacity_period_id), ('paid', '=', False)])
							paid_days_total = 0
							days_to_pay_total = 0
							total_paid = 0
							for pd in paid_days:
								pd_browse = self.pool.get('hr_payroll_cr.incapacity_period_per_day').browse(self.cr, self.uid, pd)
								payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, pd_browse.payment_per_contract_line.id)
								paid_days_total = paid_days_total + 1
								total_paid = total_paid + payment_per_contract_line.amount
							for dtp in days_to_pay:
								days_to_pay_total = days_to_pay_total + 1
							paid_days_total_per_contract += paid_days_total
							days_to_pay_total_per_contract += days_to_pay_total
							total_paid_per_contract += total_paid

						#charge the incapacity_period register whit all the information
						register = {'total_days': paid_days_total_per_contract + days_to_pay_total_per_contract, 'days_to_pay':days_to_pay_total_per_contract, 'paid_days':paid_days_total_per_contract, 'total':total_paid_per_contract, 'name':name, 'contractCode':contractCode} 
						#append the register to the answere
						answ.append(register)
		#return the list of incapacity_period	
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		

report_sxw.report_sxw('report.hr_payroll_cr.incapacityPeriodPerCompanyTotal', 'hr_payroll_cr.incapacity_period', 'addons/hr_payroll_cr/report/incapacityPeriodPerCompanyTotal.rml', parser=incapacityPeriodPerCompanyTotal, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
