# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class payrollPerContract(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.payrollPerContract'
	def __init__(self, cr, uid, name, context):
		super(payrollPerContract, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'getLoadRegisters': self._get_load_registers,
			'get_absence': self._get_absences,
			'get_rent':self._get_rent,
			'get_incapacity': self._get_incapacity,
			'get_over_salary': self._get_over_salary,
			'get_foreclosure':self._get_foreclosure,
			'get_extra_hour': self._get_extra_hour,
			'get_savings':self._get_savings,
			'get_loan':self._get_loan,
			'get_refund':self._get_refund,
			'get_deduction':self._get_deduction,
			'startDate': self._startDate,
			'endDate': self._endDate,
			'company':self._getCompany,
		})
		
	def _loadRegisters(self, form):
		company_id = form['company_id']
		start_date = form['dateStart']
		end_date = form['dateEnd']
		#to store the answer
		answ = []
		#search for all the employees of the given company
		contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id)], order='code')
		employee_list = []
		for contract_id in contracts:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			employee_list.append(contract.employee_id.id)
		employees = self.pool.get('hr.employee').search(self.cr, self.uid,[('id','in',employee_list)], order='surname1, surname2, name')
		if form['order'] == '1':
			for contract_id in contracts:
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
				contractCode = contract.code
				pay_per_contract_id = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid,[('contract','=',contract_id), ('payroll_date','>=',start_date), ('payroll_date','<=',end_date)], order='payroll_date')
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				registerAbs = {}
				for ppc_id in pay_per_contract_id:
					payment_per_contract_browse = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, ppc_id)
					registerAbs = {'detail':payment_per_contract_browse.detail, 'total_paid':payment_per_contract_browse.total_paid, 'gross_salary':payment_per_contract_browse.gross_salary, 'base_salary':contract.wage, 'employee_name':name, 'contractCode':contractCode, 'contract_id':contract_id, 'ssnid':employee.ssnid} 
			#append the register to the answer
					answ.append(registerAbs)
		else:
			for employee_id in employees:
				contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id),('employee_id','=',employee_id)], order='date_start, code')
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, employee_id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				for contract_id in contracts:
					contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
					contractCode = contract.code
					pay_per_contract_id = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid,[('contract','=',contract_id), ('payroll_date','>=',start_date), ('payroll_date','<=',end_date)], order='payroll_date')
					registerAbs = {}
					for ppc_id in pay_per_contract_id:
						payment_per_contract_browse = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, ppc_id)
						registerAbs = {'detail':payment_per_contract_browse.detail, 'total_paid':payment_per_contract_browse.total_paid, 'gross_salary':payment_per_contract_browse.gross_salary, 'base_salary':contract.wage, 'employee_name':name, 'contractCode':contractCode, 'contract_id':contract_id, 'ssnid':employee.ssnid} 
				#append the register to the answer
						answ.append(registerAbs)
		#return the list of over_salary	
		return answ
		
	
	def _get_load_registers(self, contract_id, form):
		#to store the answer
		answ = []
		payment_per_contrac = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for payment_per_contrac_id in payment_per_contrac:
			payment_per_contrac_browse = self.pool.get('hr_payroll_cr.payment_per_contract').browse(self.cr, self.uid, payment_per_contrac_id)
			register = {'id':payment_per_contrac_browse.id}
			answ.append(register)
		return answ
	
	def _get_absences(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				absence_ids = self.pool.get('hr_payroll_cr.absence').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				#load a list with the absences for that contract
				for absence_id in absence_ids:
					absence = self.pool.get('hr_payroll_cr.absence').browse(self.cr, self.uid, absence_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, absence.payment_per_contract_line.id)
					
					register = {'date':payment_per_contract_line.date, 'ab_date':absence.date, 'detail':absence.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_incapacity(self, contract_id, form):
		answ = []
		incapacity = self.pool.get('hr_payroll_cr.incapacity_period').search(self.cr, self.uid, [('contract', '=', contract_id)])
		
		for inc in incapacity:
			total = 0
			inca_browse = self.pool.get('hr_payroll_cr.incapacity_period').browse(self.cr, self.uid, inc)
			incapacity_per_day = self.pool.get('hr_payroll_cr.incapacity_period_per_day').search(self.cr, self.uid, [('incapacity_period', '=', inc), ('date','>=',form['dateStart']), ('date','<=',form['dateEnd']), ('paid', '=', True)])
			if not incapacity_per_day == []:
				for inc_per_day in incapacity_per_day:
					inc_per_browse = self.pool.get('hr_payroll_cr.incapacity_period_per_day').browse(self.cr, self.uid, inc_per_day)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, inc_per_browse.payment_per_contract_line.id)
					total = total + payment_per_contract_line.amount
				register = {'date':inca_browse.start_date, 'detail':inca_browse.detail, 'amount':total}
				answ.append(register)
		return answ	
	
	def _get_over_salary(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				over_salary_ids = self.pool.get('hr_payroll_cr.over_salary_payment').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				#load a list with the abbsences for that contract
				for over_salary_id in over_salary_ids:
					over_salary = self.pool.get('hr_payroll_cr.over_salary_payment').browse(self.cr, self.uid, over_salary_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, over_salary.payment_per_contract_line.id)
					register = {'date':over_salary.date, 'detail':over_salary.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_extra_hour(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				extra_hour_ids = self.pool.get('hr_payroll_cr.extra_hour').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				#load a list with the abbsences for that contract
				for extra_hour_id in extra_hour_ids:
					extra_hour = self.pool.get('hr_payroll_cr.extra_hour').browse(self.cr, self.uid, extra_hour_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, extra_hour.payment_per_contract_line.id)
					register = {'date':extra_hour.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_savings(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				saving_ids = self.pool.get('hr_payroll_cr.saving_quota').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				for saving_id in saving_ids:
					saving_browse = self.pool.get('hr_payroll_cr.saving_quota').browse(self.cr, self.uid, saving_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, saving_browse.payment_per_contract_line.id)
					register = {'date':payment_per_contract_line.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_loan(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				loan_ids = self.pool.get('hr_payroll_cr.loan_payment').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				for loan_id in loan_ids:
					loan_browse = self.pool.get('hr_payroll_cr.loan_payment').browse(self.cr, self.uid, loan_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, loan_browse.payment_per_contract_line.id)
					register = {'date':payment_per_contract_line.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_foreclosure(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				foreclosure_ids = self.pool.get('hr_payroll_cr.foreclosure_payment').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				for foreclosure_id in foreclosure_ids:
					foreclosure_browse = self.pool.get('hr_payroll_cr.foreclosure_payment').browse(self.cr, self.uid, foreclosure_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, foreclosure_browse.payment_per_contract_line.id)
					register = {'date':payment_per_contract_line.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_deduction(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				deduction_ids = self.pool.get('hr_payroll_cr.deduction_line').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				for deduction_id in deduction_ids:
					deduction_browse = self.pool.get('hr_payroll_cr.deduction_line').browse(self.cr, self.uid, deduction_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, deduction_browse.payment_per_contract_line.id)
					register = {'date':payment_per_contract_line.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_refund(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				refund_ids = self.pool.get('hr_payroll_cr.refund').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				for refund_id in refund_ids:
					refund_browse = self.pool.get('hr_payroll_cr.refund').browse(self.cr, self.uid, refund_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, refund_browse.payment_per_contract_line.id)
					register = {'date':payment_per_contract_line.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _get_rent(self, contract_id, form):
		answ = []
		payment_per_contract = self.pool.get('hr_payroll_cr.payment_per_contract').search(self.cr, self.uid, [('contract','=',contract_id), ('payroll_date','>=',form['dateStart']), ('payroll_date','<=',form['dateEnd'])])
		for ppc in payment_per_contract:
			payment_per_contract_lines_ids = self.pool.get('hr_payroll_cr.payment_per_contract_line').search(self.cr, self.uid, [('payment_per_contract','=',ppc)])
			for ppcli in payment_per_contract_lines_ids:
				rent_ids = self.pool.get('hr_payroll_cr.rent_tax').search(self.cr, self.uid, [('payment_per_contract_line','=',ppcli)])
				for rent_id in rent_ids:
					rent_browse = self.pool.get('hr_payroll_cr.rent_tax').browse(self.cr, self.uid, rent_id)
					payment_per_contract_line = self.pool.get('hr_payroll_cr.payment_per_contract_line').browse(self.cr, self.uid, rent_browse.payment_per_contract_line.id)
					register = {'date':payment_per_contract_line.date, 'detail':payment_per_contract_line.detail, 'amount':payment_per_contract_line.amount}
					answ.append(register)
		return answ
	
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
	
	def _getCompany(self, form):
		company = form['company_id']
		return company
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		

report_sxw.report_sxw('report.hr_payroll_cr.payrollPerContract', 'hr_payroll_cr.payroll', 'addons/hr_payroll_cr/report/payrollPerContract.rml', parser=payrollPerContract, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
