# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
import tools
from tools import debug

class permitPerCompanyTotal(report_sxw.rml_parse):
	_name = 'report.hr_payroll_cr.permitPerCompanyTotal'
	def __init__(self, cr, uid, name, context):
		super(permitPerCompanyTotal, self).__init__(cr, uid, name, context=context)
		self.localcontext.update({ 
			'time': time,
			'loadRegisters':self._loadRegisters,
			'startDate': self._startDate,
			'endDate': self._endDate,
			'get_company': self._get_company,
		})
		
	def _loadRegisters(self, form):
		company_id = form['company_id']
		#to store the answer
		answ = []
		#search for all the employees of the given company
		contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id)], order='code')
		employee_list = []
		for contract_id in contracts:
			contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
			employee_list.append(contract.employee_id.id)
		employees = self.pool.get('hr.employee').search(self.cr, self.uid,[('id','in',employee_list)], order='surname1, surname2, name')
		permits = self.pool.get('hr_payroll_cr.permit').search(self.cr, self.uid,[('contract','in',contracts)], order='start_date')
		if form['order'] == '1':
			for contract_id in contracts:
				contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
				contractCode = contract.code			
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, contract.employee_id.id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
				
				permits = self.pool.get('hr_payroll_cr.permit').search(self.cr, self.uid,[('contract','=',contract_id)], order='start_date')
				#for each permit per contract
				for permit_id in permits:
					permit = self.pool.get('hr_payroll_cr.permit').browse(self.cr, self.uid, permit_id)#loads the permit register to store the permit data
					#charge the permit register with all the information
					permit_per_day = self.pool.get('hr_payroll_cr.permit_per_day').search(self.cr, self.uid, [('permit', '=', permit_id), ('date', '>=', form['dateStart']), ('date', '<=', form['dateEnd'])])
					days_total = 0
					for ppd in permit_per_day:
						ppd_browse = self.pool.get('hr_payroll_cr.permit_per_day').browse(self.cr, self.uid, ppd)
						days_total += 1
					if (days_total != 0):
						register = {'name':name, 'contractCode':contractCode, 'count': days_total} 
						#append the register to the answer
						answ.append(register)
		elif form['order'] == '2':
			for employee_id in employees:
				contracts = self.pool.get('hr.contract').search(self.cr, self.uid,[('company_id','=',company_id),('employee_id','=',employee_id)], order='date_start, code')
				employee = self.pool.get('hr.employee').browse(self.cr, self.uid, employee_id) #loads the employee register to store the employee name
				name = employee.surname1 + ' ' + employee.surname2 + ' ' + employee.name
							
				for contract_id in contracts:
					contract = self.pool.get('hr.contract').browse(self.cr, self.uid, contract_id)#loads the contract register to store the contract name
					contractCode = contract.code
					permits = self.pool.get('hr_payroll_cr.permit').search(self.cr, self.uid,[('contract','=',contract_id)], order='start_date')
					#for each permit per contract
					for permit_id in permits:
						permit = self.pool.get('hr_payroll_cr.permit').browse(self.cr, self.uid, permit_id)#loads the permit register to store the permit data
						#charge the permit register whit all the information
						permit_per_day = self.pool.get('hr_payroll_cr.permit_per_day').search(self.cr, self.uid, [('permit', '=', permit_id), ('date', '>=', form['dateStart']), ('date', '<=', form['dateEnd'])])
						days_total = 0
						for ppd in permit_per_day:
							ppd_browse = self.pool.get('hr_payroll_cr.permit_per_day').browse(self.cr, self.uid, ppd)
							days_total += 1
						if (days_total != 0):
							register = {'name':name, 'contractCode':contractCode, 'count':days_total} 
							#append the register to the answer
							answ.append(register)
		#return the list of permit	
		return answ
		
	def _startDate(self, form):
		date = form['dateStart']
		return str(date)
		
	def _endDate(self, form):
		date = form['dateEnd']
		return str(date)
		
	def _get_company(self, form):
		company_id = form['company_id']
		debug(company_id)
		name = self.pool.get('res.company').browse(self.cr, self.uid, company_id)
		debug(name)
		debug(name.name)
		return name.name
		

report_sxw.report_sxw('report.hr_payroll_cr.permitPerCompanyTotal', 'hr_payroll_cr.permit', 'addons/hr_payroll_cr/report/permitPerCompanyTotal.rml', parser=permitPerCompanyTotal, header=False)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
